@@ -10,11 +10,10 @@
 #include "update_matrices/data.h"
 
 
-static char* test_form_KKT() {
+static const char* test_form_KKT() {
   update_matrices_sols_data *data;
   c_float sigma, *rho_vec, *rho_inv_vec;
   c_int   m, *PtoKKT, *AtoKKT, *Pdiag_idx, Pdiag_n;
-  csc    *Ptriu;
   csc    *KKT;
 
   // Load problem data
@@ -23,22 +22,19 @@ static char* test_form_KKT() {
   // Define rho_vec and sigma to form KKT
   sigma       = data->test_form_KKT_sigma;
   m           = data->test_form_KKT_A->m;
-  rho_vec     = c_calloc(m, sizeof(c_float));
-  rho_inv_vec = c_calloc(m, sizeof(c_float));
+  rho_vec     = (c_float*) c_calloc(m, sizeof(c_float));
+  rho_inv_vec = (c_float*) c_calloc(m, sizeof(c_float));
   vec_add_scalar(rho_vec, data->test_form_KKT_rho, m);
   vec_ew_recipr(rho_vec, rho_inv_vec, m);
 
   // Allocate vectors of indices
-  PtoKKT = c_malloc((data->test_form_KKT_Pu->p[data->test_form_KKT_Pu->n]) *
+  PtoKKT = (c_int*) c_malloc((data->test_form_KKT_Pu->p[data->test_form_KKT_Pu->n]) *
                     sizeof(c_int));
-  AtoKKT = c_malloc((data->test_form_KKT_A->p[data->test_form_KKT_A->n]) *
+  AtoKKT = (c_int*) c_malloc((data->test_form_KKT_A->p[data->test_form_KKT_A->n]) *
                     sizeof(c_int));
 
-  // Form upper triangular part in P
-  Ptriu = csc_to_triu(data->test_form_KKT_P);
-
   // Form KKT matrix storing the index vectors
-  KKT = form_KKT(Ptriu,
+  KKT = form_KKT(data->test_form_KKT_Pu,
                  data->test_form_KKT_A,
                  0,
                  sigma,
@@ -67,7 +63,6 @@ static char* test_form_KKT() {
   // Cleanup
   clean_problem_update_matrices_sols_data(data);
   c_free(Pdiag_idx);
-  csc_spfree(Ptriu);
   csc_spfree(KKT);
   c_free(rho_vec);
   c_free(rho_inv_vec);
@@ -76,12 +71,13 @@ static char* test_form_KKT() {
   return 0;
 }
 
-static char* test_update() {
+static const char* test_update() {
   c_int i, nnzP, nnzA;
   update_matrices_sols_data *data;
   OSQPData *problem;
   OSQPWorkspace *work;
   OSQPSettings  *settings;
+  c_int exitflag;
 
   // Update matrix P
   c_int *Px_new_idx;
@@ -93,13 +89,13 @@ static char* test_update() {
   data = generate_problem_update_matrices_sols_data();
 
   // Generate first problem data
-  problem    = c_malloc(sizeof(OSQPData));
-  problem->P = data->test_solve_P;
+  problem    = (OSQPData*) c_malloc(sizeof(OSQPData));
+  problem->P = data->test_solve_Pu;
   problem->q = data->test_solve_q;
   problem->A = data->test_solve_A;
   problem->l = data->test_solve_l;
   problem->u = data->test_solve_u;
-  problem->n = data->test_solve_P->n;
+  problem->n = data->test_solve_Pu->n;
   problem->m = data->test_solve_A->m;
 
 
@@ -112,10 +108,10 @@ static char* test_update() {
   settings->verbose  = 1;
 
   // Setup workspace
-  work = osqp_setup(problem, settings);
+  exitflag = osqp_setup(&work, problem, settings);
 
   // Setup correct
-  mu_assert("Update matrices: original problem, setup error!", work != OSQP_NULL);
+  mu_assert("Update matrices: original problem, setup error!", exitflag == 0);
 
   // Solve Problem
   osqp_solve(work);
@@ -137,9 +133,9 @@ static char* test_update() {
 
   // Update P
   nnzP       = data->test_solve_Pu->p[data->test_solve_Pu->n];
-  Px_new_idx = c_malloc(nnzP * sizeof(c_int)); // Generate indices going from
-                                               // beginning to end of P
+  Px_new_idx = (c_int*) c_malloc(nnzP * sizeof(c_int));
 
+  // Generate indices going from beginning to end of P
   for (i = 0; i < nnzP; i++) {
     Px_new_idx[i] = i;
   }
@@ -150,57 +146,108 @@ static char* test_update() {
   osqp_solve(work);
 
   // Compare solver statuses
-  mu_assert("Update matrices: problem with P updated, error in solver status!",
+  mu_assert("Update matrices: problem with updating P, error in solver status!",
             work->info->status_val == data->test_solve_P_new_status);
 
   // Compare primal solutions
-  mu_assert("Update matrices: problem with P updated, error in primal solution!",
+  mu_assert("Update matrices: problem with updating P, error in primal solution!",
             vec_norm_inf_diff(work->solution->x, data->test_solve_P_new_x,
                               data->n) < TESTS_TOL);
 
   // Compare dual solutions
-  mu_assert("Update matrices: problem with P updated, error in dual solution!",
+  mu_assert("Update matrices: problem with updating P, error in dual solution!",
             vec_norm_inf_diff(work->solution->y, data->test_solve_P_new_y,
                               data->m) < TESTS_TOL);
 
+  // Cleanup and setup workspace
+  osqp_cleanup(work);
+  exitflag = osqp_setup(&work, problem, settings);
+
+
+  // Update P (all indices)
+  osqp_update_P(work, data->test_solve_Pu_new->x, OSQP_NULL, nnzP);
+
+  // Solve Problem
+  osqp_solve(work);
+
+  // Compare solver statuses
+  mu_assert("Update matrices: problem with updating P (all indices), error in solver status!",
+            work->info->status_val == data->test_solve_P_new_status);
+
+  // Compare primal solutions
+  mu_assert("Update matrices: problem with updating P (all indices), error in primal solution!",
+            vec_norm_inf_diff(work->solution->x, data->test_solve_P_new_x,
+                              data->n) < TESTS_TOL);
+
+  // Compare dual solutions
+  mu_assert("Update matrices: problem with updating P (all indices), error in dual solution!",
+            vec_norm_inf_diff(work->solution->y, data->test_solve_P_new_y,
+                              data->m) < TESTS_TOL);
+
+  // Cleanup and setup workspace
+  osqp_cleanup(work);
+  exitflag = osqp_setup(&work, problem, settings);
+
 
   // Update A
   nnzA       = data->test_solve_A->p[data->test_solve_A->n];
-  Ax_new_idx = c_malloc(nnzA * sizeof(c_int)); // Generate indices going from
-                                               // beginning to end of P
+  Ax_new_idx = (c_int*) c_malloc(nnzA * sizeof(c_int));
 
+  // Generate indices going from beginning to end of A
   for (i = 0; i < nnzA; i++) {
     Ax_new_idx[i] = i;
   }
 
-  // Cleanup and setup workspace
-  osqp_cleanup(work);
-  work = osqp_setup(problem, settings);
-
   osqp_update_A(work, data->test_solve_A_new->x, Ax_new_idx, nnzA);
 
   // Solve Problem
   osqp_solve(work);
 
   // Compare solver statuses
-  mu_assert("Update matrices: problem with A updated, error in solver status!",
+  mu_assert("Update matrices: problem with updating A, error in solver status!",
             work->info->status_val == data->test_solve_A_new_status);
 
   // Compare primal solutions
-  mu_assert("Update matrices: problem with A updated, error in primal solution!",
+  mu_assert("Update matrices: problem with updating A, error in primal solution!",
             vec_norm_inf_diff(work->solution->x, data->test_solve_A_new_x,
                               data->n) < TESTS_TOL);
 
   // Compare dual solutions
-  mu_assert("Update matrices: problem with A updated, error in dual solution!",
+  mu_assert("Update matrices: problem with updating A, error in dual solution!",
+            vec_norm_inf_diff(work->solution->y, data->test_solve_A_new_y,
+                              data->m) < TESTS_TOL);
+
+  // Cleanup and setup workspace
+  osqp_cleanup(work);
+  exitflag = osqp_setup(&work, problem, settings);
+
+
+  // Update A (all indices)
+  osqp_update_A(work, data->test_solve_A_new->x, OSQP_NULL, nnzA);
+
+  // Solve Problem
+  osqp_solve(work);
+
+  // Compare solver statuses
+  mu_assert("Update matrices: problem with updating A (all indices), error in solver status!",
+            work->info->status_val == data->test_solve_A_new_status);
+
+  // Compare primal solutions
+  mu_assert("Update matrices: problem with updating A (all indices), error in primal solution!",
+            vec_norm_inf_diff(work->solution->x, data->test_solve_A_new_x,
+                              data->n) < TESTS_TOL);
+
+  // Compare dual solutions
+  mu_assert("Update matrices: problem with updating A (all indices), error in dual solution!",
             vec_norm_inf_diff(work->solution->y, data->test_solve_A_new_y,
                               data->m) < TESTS_TOL);
 
 
   // Cleanup and setup workspace
   osqp_cleanup(work);
-  work = osqp_setup(problem, settings);
+  exitflag = osqp_setup(&work, problem, settings);
 
+  // Update P and A
   osqp_update_P_A(work, data->test_solve_Pu_new->x, Px_new_idx, nnzP,
                   data->test_solve_A_new->x, Ax_new_idx, nnzA);
 
@@ -209,18 +256,47 @@ static char* test_update() {
 
   // Compare solver statuses
   mu_assert(
-    "Update matrices: problem with P and A updated, error in solver status!",
+    "Update matrices: problem with updating P and A, error in solver status!",
     work->info->status_val == data->test_solve_P_A_new_status);
 
   // Compare primal solutions
   mu_assert(
-    "Update matrices: problem with P and A updated, error in primal solution!",
+    "Update matrices: problem with updating P and A, error in primal solution!",
     vec_norm_inf_diff(work->solution->x, data->test_solve_P_A_new_x,
                       data->n) < TESTS_TOL);
 
   // Compare dual solutions
   mu_assert(
-    "Update matrices: problem with P and A updated, error in dual solution!",
+    "Update matrices: problem with updating P and A, error in dual solution!",
+    vec_norm_inf_diff(work->solution->y, data->test_solve_P_A_new_y,
+                      data->m) < TESTS_TOL * TESTS_TOL);
+
+  // Cleanup and setup workspace
+  osqp_cleanup(work);
+  exitflag = osqp_setup(&work, problem, settings);
+
+
+  // Update P and A (all indices)
+  osqp_update_P_A(work, data->test_solve_Pu_new->x, OSQP_NULL, nnzP,
+                  data->test_solve_A_new->x, OSQP_NULL, nnzA);
+
+  // Solve Problem
+  osqp_solve(work);
+
+  // Compare solver statuses
+  mu_assert(
+    "Update matrices: problem with updating P and A (all indices), error in solver status!",
+    work->info->status_val == data->test_solve_P_A_new_status);
+
+  // Compare primal solutions
+  mu_assert(
+    "Update matrices: problem with updating P and A (all indices), error in primal solution!",
+    vec_norm_inf_diff(work->solution->x, data->test_solve_P_A_new_x,
+                      data->n) < TESTS_TOL);
+
+  // Compare dual solutions
+  mu_assert(
+    "Update matrices: problem with updating P and A (all indices), error in dual solution!",
     vec_norm_inf_diff(work->solution->y, data->test_solve_P_A_new_y,
                       data->m) < TESTS_TOL * TESTS_TOL);
 
@@ -238,7 +314,7 @@ static char* test_update() {
 
 #ifdef ENABLE_MKL_PARDISO
 static char* test_update_pardiso() {
-  c_int i, nnzP, nnzA;
+  c_int i, nnzP, nnzA, exitflag;
   update_matrices_sols_data *data;
   OSQPData *problem;
   OSQPWorkspace *work;
@@ -255,12 +331,12 @@ static char* test_update_pardiso() {
 
   // Generate first problem data
   problem    = c_malloc(sizeof(OSQPData));
-  problem->P = data->test_solve_P;
+  problem->P = data->test_solve_Pu;
   problem->q = data->test_solve_q;
   problem->A = data->test_solve_A;
   problem->l = data->test_solve_l;
   problem->u = data->test_solve_u;
-  problem->n = data->test_solve_P->n;
+  problem->n = data->test_solve_Pu->n;
   problem->m = data->test_solve_A->m;
 
 
@@ -274,10 +350,10 @@ static char* test_update_pardiso() {
   settings->linsys_solver = MKL_PARDISO_SOLVER;
 
   // Setup workspace
-  work = osqp_setup(problem, settings);
+  exitflag = osqp_setup(&work, problem, settings);
 
   // Setup correct
-  mu_assert("Update matrices: original problem, setup error!", work != OSQP_NULL);
+  mu_assert("Update matrices: original problem, setup error!", exitflag == 0);
 
   // Solve Problem
   osqp_solve(work);
@@ -337,7 +413,7 @@ static char* test_update_pardiso() {
 
   // Cleanup and setup workspace
   osqp_cleanup(work);
-  work = osqp_setup(problem, settings);
+  exitflag = osqp_setup(&work, problem, settings);
 
   osqp_update_A(work, data->test_solve_A_new->x, Ax_new_idx, nnzA);
 
@@ -361,7 +437,7 @@ static char* test_update_pardiso() {
 
   // Cleanup and setup workspace
   osqp_cleanup(work);
-  work = osqp_setup(problem, settings);
+  exitflag = osqp_setup(&work, problem, settings);
 
   osqp_update_P_A(work, data->test_solve_Pu_new->x, Px_new_idx, nnzP,
                   data->test_solve_A_new->x, Ax_new_idx, nnzA);
@@ -399,7 +475,7 @@ static char* test_update_pardiso() {
 }
 #endif
 
-static char* test_update_matrices()
+static const char* test_update_matrices()
 {
   mu_run_test(test_form_KKT);
   mu_run_test(test_update);
